/* crypto/sha/sha1test.c */
/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
 * All rights reserved.
 *
 * This package is an SSL implementation written
 * by Eric Young (eay@cryptsoft.com).
 * The implementation was written so as to conform with Netscapes SSL.
 * 
 * This library is free for commercial and non-commercial use as long as
 * the following conditions are aheared to.  The following conditions
 * apply to all code found in this distribution, be it the RC4, RSA,
 * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
 * included with this distribution is covered by the same copyright terms
 * except that the holder is Tim Hudson (tjh@cryptsoft.com).
 * 
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.
 * If this package is used in a product, Eric Young should be given attribution
 * as the author of the parts of the library used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    "This product includes cryptographic software written by
 *     Eric Young (eay@cryptsoft.com)"
 *    The word 'cryptographic' can be left out if the rouines from the library
 *    being used are not cryptographic related :-).
 * 4. If you include any Windows specific code (or a derivative thereof) from 
 *    the apps directory (application code) you must include an acknowledgement:
 *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
 * 
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>



//#include "../e_os.h"

#include <openssl/evp.h>
#include <openssl/sha.h>

#ifdef CHARSET_EBCDIC
#include <openssl/ebcdic.h>
#endif

#undef SHA_0 /* FIPS 180 */
#define  SHA_1 /* FIPS 180-1 */

static char *pt(unsigned char *md);
int main(int argc, char *argv[])
{
	if (argc!=3)
	{
		printf("usage: %s key_length(in number of 20 bytes) iterations\n", argv[0]);
		return 0;
	}	
		
	int i,j;
	int num, iteration;
	num=atoi(argv[1]);
	iteration=atoi(argv[2]);
	EVP_MD_CTX c;
	unsigned char md[SHA_DIGEST_LENGTH];
	unsigned char res[SHA_DIGEST_LENGTH];
	unsigned char* byte;
  unsigned char* tmp;
  clock_t c0, c1;
  double cputime;
	char *p;

 

	EVP_MD_CTX_init(&c);

	unsigned long value=3;
	unsigned char* input=(unsigned char*)&value;

	EVP_Digest(input,sizeof(unsigned long),md,NULL,EVP_sha1(), NULL);
	unsigned long length=num*SHA_DIGEST_LENGTH/2;
	byte=(unsigned char*)malloc(length*sizeof(unsigned char));	
	tmp=byte;
	for (i=0;i<num;i++)
	{
		memcpy(tmp, md, SHA_DIGEST_LENGTH/2);
		tmp+=SHA_DIGEST_LENGTH/2;
	}

	c0 = clock();
	for (i=0;i<iteration;i++)
			EVP_Digest(byte,length,res,NULL,EVP_sha1(), NULL);
	c1 = clock();
	cputime = (double)(c1 - c0)/(double)(CLOCKS_PER_SEC );
	cputime = cputime/(double)iteration;
	printf ("\tElapsed CPU time test:   %f  sec\n",cputime);

	p=pt(res);
	printf("got %s\n",p);
	EVP_MD_CTX_cleanup(&c);

		
	EVP_DigestInit_ex(&c,EVP_sha1(), NULL);
	c0 = clock();
	for (j=0;j<iteration;j++)
	{	
		for (i=0; i<num; i++)
			EVP_DigestUpdate(&c,md,SHA_DIGEST_LENGTH/2);
		EVP_DigestFinal_ex(&c,res,NULL);
	}
	c1 = clock();
	cputime = (double)(c1 - c0)/(double)(CLOCKS_PER_SEC );
	cputime = cputime/(double)iteration;
	printf ("\tElapsed CPU time test:   %f  sec\n",cputime);
	p=pt(res);
	printf("got %s\n",p);


	EVP_MD_CTX_cleanup(&c);
	return(0);
	}

static char *pt(unsigned char *md)
	{
	int i;
	static char buf[80];
	//test 1
	for (i=0; i<SHA_DIGEST_LENGTH; i++)
		sprintf(&(buf[i*2]),"%02x",md[i]);
	return(buf);
	}

